#ifndef C_TORUS_H
#define C_TORUS_H

#include <vector>
#include "SPlane3.h"



/***
 *** This class represents a "circle torus" in 3-dimensional space. It consists of a number of num_circles circles
 *** that are pre-images of points on the two-sphere under the hopf map.
 ***/


// Achtung: Zum Teil schon erledigt!
// TODO: Als private members eine Ebene sect_plane hinzufuegen.
//       Ausserdem eine Funktion recalculate(), die bei Bedarf die Kreise des Torus neu berechnet.
//       Diese wird erst vor dem Zeichnen aufgerufen.
//       Die Variablen sect_plane und num_circles koennen ueber entsprechende Funktionen gesetzt werden.
//
//       evtl. eine Variable "coloring" einfuehren, die die Faerbung der Kreise des Torus festlegt,
//             z.B. UNI oder SMOOTH oder so;
//       

enum drawmethod {CIRCS, FOLIATED, MESH, SOLID};
enum colormodel {UNI, SMOOTH};
// TODO: remove later:
enum cycl_param {PARAM_A, PARAM_B, PARAM_D};

class CTorus {
 public:
    CTorus();
    // This function calculates num points on the intersecting circle of the circplane with S^2 as well as
    // their preimages under the hopf map. These preimages are collected in the torus
    //    CTorus(SPlane3 circplane, unsigned int num, drawmethod=CIRCS, colormodel colmod=UNI);
    CTorus(SPlane3 circplane, unsigned int num, drawmethod=CIRCS, colormodel colmod=UNI);

    SPlane3 get_sect_plane();
    void set_sect_plane(SPlane3);
    pair<double, double> get_defining_angles();
    //    void set_defining_angles(pair<double, double>); // TODO: Evtl. nicht ntig


    colormodel get_coloring();
    void set_coloring(colormodel colmod);
    drawmethod get_drawmethod();
    void set_drawmethod(drawmethod);
    double* get_color();
    void set_color(double R, double G, double B);
    Vector3 get_major_axis(); /* returns major axis of the cyclide */

    Vector3 calc_major_axis_from_extremal_circles();
    Vector3 calc_major_axis_from_smallest_and_biggest_circle();
    pair<pair<Vector3, Vector3>, pair<Vector3, Vector3> > calc_P_Q_R_S();

    unsigned int get_num_circles();
    void set_num_circles(unsigned int number);
    void recalc(); // Calculates the circles of the torus as preimages of points on a circle C.
                   // C is the intersecting circle of sect_plane and S^2
    // Calculates the image of each circle of the torus under inversion with respect to
    // the unit sphere around "zentrum". This is for testing if the deformed torus is indeed a Dupin cyclide
    // TODO: spter noch bentigt?
    CTorus invert(Vector3 zentrum=Vector3(0,0,0));
    void draw(); // serves as a unified interface for the various drawing methods (the behaviour is set by the private variable "dmeth")
    void circ_draw(); // draws the circles of the torus
    void fol_draw(); // vorerst nur ein Test...
    void fol_draw_other(); // vorerst nur ein Test (mit Bezier-Flaechen statt NURBS)...
    // Wurde hier vielleicht eine Funktion entfernt????
    // The following function is the same as "draw_circle_strip" below, but the circles given as the first two arguments
    // are not given as "circles". Instead, their indices in the vector "torus" are specified.
    double draw_circle_strip(unsigned int circle_no1,unsigned int circle_no2, unsigned int num, double t, bool solid=true);
    void surf_draw(); // draws the torus as a connected surface
    void surf_draw2(); // test method to draw the surface of the torus as a series of OpenGL evaluator surfaces
    void surf_draw3_old(); // yet another test method to draw the surface of the torus as a series of OpenGL evaluator surfaces
    void surf_draw3();     // yet another test method to draw the surface of the torus as a series of OpenGL evaluator surfaces
    void surf_nurbs_draw(Vector3 &point, bool one=true, bool two=true);
                            // draws the torus as a single NURBS surface; THIS IS NOT YET FULLY IMPLEMENTED (if ever...)
                            // so far, it only calculates some data related to the torus...
                            // TODO: remove the Vector3 and the boolean arguments. The serve only for the testing
                            // function "get_four_points" !!!!
    pair<Circle3d, Circle3d> calc_smallest_and_biggest_circle(); // returns the smallest and the 
                     // biggest circle (in this order) *within* the cyclide (NOT the "extremal circles"
    Circle3d get_first_circle(); // for testing purposes only; TODO: must be removed later
    double calc_cycl_parameters(); // for testing purposes only; TODO: should be removed later
    // calculates the four intersecting points of the "small" and the "big" circle of the cyclide (viewed from aside)
    // with the x,y-plane. From these points the radii of these circles as well as the corrsponding parameters a,b and d
    // of the bezier surface patches can be calculated and the requested parameter is returned

    //TODO: To be made private (why is this in the public section????)
    vector<Circle3d> torus; // this vector contains the circles which form the final "torus" in 3-dimensional space

    GLfloat ctrlpts_aff00[3][3][4];
    GLfloat ctrlpts_aff01[3][3][4];
    GLfloat ctrlpts_aff02[3][3][4];
    GLfloat ctrlpts_aff03[3][3][4];

    static const int texImageWidth=64;
    static const int texImageHeight=64;
    GLubyte texImage[3*texImageWidth*texImageHeight];
    GLfloat texpts[2][2][2];


    // TODO: Komplett in die Klasse einbauen;
    //    void recalc_cycl_parameters(double rad0, double rad1, double rad2);


 private:
    // TODO: evtl. vector<Circle3d*>, also Pointer verwenden?
    // TODO: 
    CTorus(vector<Circle3d> *); // for internal purposes only!!!
    bool recalcable; // a flag indicating whether the torus is an inversion torus. If so, the recalc() function
                     // must not be called before drawing the torus. Otherwise, the result is wrong (because the
                     // torus is *not* the image of a point on S^2 and can therefore not be calculated as one)
                     // TODO: Ist das noch ntig? Wird es berhaupt verwendet?
    vector<Vector3> points_on_S2; // this vector contains the points on S^2 which yield the circles stored in torus
    unsigned int num_circles;
    SPlane3 sect_plane; // this is the plane whose intersection with S^2 yields the "pre_circle" of the torus
    Vector3 major_axis; // the "major axis" of the cyclide (i.e. its "line of symmetry")
    //    double alpha, beta; /* the angles that define the torus' shape and orientation */
    double xyangle; //  amount by which the cyclide is turned around the z-axis
    double skewangle; // amount by which the cyclide is turned around its "major axis" with respect to the x,y-plane; also determines
                      // the degree of deformation of the torus
    double alpha, beta, gamma; /* the angles that determine the shape and orientation of the torus */
                               // TODO: What is gamma???

    double defining_alpha, defining_beta; // These are the angles between the normal vector of the defining plane 
                                          // and the coordinate axes

    // Caution: These are NOT the angles which the normal vector of the defining plane include with the coordinate axis
    bool WRAP; /* indicates if the cyclide is "wrapped around" */
    void set_wrap(); // if torus data has changed, reset WRAP accordingly
    drawmethod dmeth;
    colormodel coloring;
    double col_vec[3]; // determines the color of the torus; only relevant if coloring==UNI
    /*
    double ctrlpts_aff00[3][3][4];
    double ctrlpts_aff01[3][3][4];
    double ctrlpts_aff02[3][3][4];
    double ctrlpts_aff03[3][3][4];
    */

 public:
    void calc_control_point_parameters(); /* calculates the parameters for the control points of the bezier patches of the torus */
    // TODO: Werden diese Kontrollpunkte momentan innerhalb oder auerhalb der Klasse gespeichert?

    // The following function calculates the normal vectors on the torus in num points which are equally distributed
    // on the circle of the torus whose index is circ_on_torus
    vector<Vector3> get_normals(unsigned int circ_index, unsigned int num, vector<Vector3> &points);

};
double draw_circle_strip(Circle3d &circle1, Circle3d &circle2, unsigned int num, double t, bool solid=true);

/*
Vector3 x_vect(1,0,0);
Vector3 y_vect(0,1,0);
Vector3 z_vect(0,0,1);
*/

#endif
