#ifndef GTK_TORUSPARAMSFRAME_H
#define GTK_TORUSPARAMSFRAME_H

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <gtk--/box.h>
#include <gtk--/entry.h>
#include <gtk--/table.h>
#include <gtk--/label.h>
#include <gtk--/frame.h>
#include <gtk--/adjustment.h>
#include <gtk--/scale.h>
#include <gtk--/range.h>
#include <gtk--/spinbutton.h>
#include <gtk--/radiobutton.h>

#include <vector>
#include <list>

#include "Vector3.h"
#include "QuatMath.h"
#include "CTorus.h"

/* The entries are numbered in the following order:
   1 - normalx
   2 - normaly
   3 - normalz
   4 - normalalpha
   5 - normalbeta
   6 -distance
   The first three provide the normal vector's coordinates, #4 and #5 are the angles between the normal and the axes,
   and #6 is the distance of the intersecting plane from the origin.
*/

class TorusParamsFrame : public Gtk::Frame {
 public:
  vector<Gtk::Label* > ll;
  //  vector<Gtk::Entry* > el;

  Gtk::Entry normalx;
  Gtk::Entry normaly;
  Gtk::Entry normalz;

  //  Gtk::Label label1;
  //  Gtk::Label label2;

  Gtk::Frame normal_frame;
  Gtk::Frame distance_frame;

  Gtk::Frame draw_props_frame;
  Gtk::Frame draw_method_frame;
  Gtk::RadioButton dp_rb_circs, dp_rb_foliation, dp_rb_mesh, dp_rb_solid;
  Gtk::Frame slicing_frame; /* TODO: new name for this frame */

  void radio_button_pressed(drawmethod meth); /* to make sure that the buttons were clicked */


  Gtk::HBox hbox1;
  Gtk::HBox hbox2;
  Gtk::HBox hbox3; /* for the distance SpinButton and Scale widgets */
  Gtk::HBox hbox4; /* for the drawing properties widgets */
  Gtk::VBox vbox1;
  Gtk::VBox vbox2;
  Gtk::VBox vbox3;
  Gtk::VBox vbox4;
  Gtk::VBox vbox5; /* for the drawing method radio button group */
  Gtk::VBox vbox6; /* for the slicing coarseness widgets */

  Gtk::Table table;

  Gtk::Adjustment alpha_adjust;
  Gtk::Adjustment beta_adjust;
  Gtk::SpinButton alpha_spin;
  Gtk::SpinButton beta_spin;
  Gtk::HScale alpha_scale;
  Gtk::HScale beta_scale;

  Gtk::Adjustment distance_adjust;
  Gtk::SpinButton distance_spin;
  Gtk::HScale distance_scale;

  Gtk::Adjustment slicing_adjust;
  Gtk::SpinButton slicing_spin;
  Gtk::HScale slicing_scale;


  /* methods which provide internal write access to the normal vector and the distance of the intersecting plane */
  // TODO: Make them private??
  // TODO: Are they really needed???
  void set_normal(Vector3 normal_vec);
  void set_angles(double alpha, double beta);
  void set_distance(double dist);
  void set_draw_method(drawmethod);

 public:
  TorusParamsFrame();
  //  void on_entry_activated(bool coord_active);
  void test_connect_func(); // TODO: Can this one be deleted? What did it once test?
  void on_angle_changed();
  void on_coord_changed();

  /** rising when activated and providing vector index */
  // TODO: Is this one needed??
  //  SigC::Signal1<void, int> entry_activated;

  /* methods which provide access to the normal vector and the distance of the intersecting plane */
  Vector3 get_normal(bool from_angle=true); /* get normal from angle or from coordinates */
  double get_distance();
  drawmethod get_draw_method();
  int get_slicing_number();
  void set_slicing_number(int);


  ////  static bool torus_data_should_be_updated;

  /* get the text value of entry with index i */
  //  string get_entry( int i) { return el[i]->get_text(); }

  /* set the text value of entry with index i */
  //  void set_entry( int i, string s) { el[i]->set_text(s); }
};

/*
  vector<Gtk::Label* > ll;
  vector<Gtk::Entry* > el;
  Gtk::HBox hbox;
*/

#endif
