#include "HypersphereViewer.h"

using SigC::bind;
using SigC::slot;


HypersphereViewer::HypersphereViewer():
  GenericViewer("Hypersphere Viewer"),
  add_torus_window(),
  params_window_button("Parameter window"),
  delete_torus_button("Delete Torus")
{
  using namespace Gtk::Toolbar_Helpers;
  GenericViewer::toolbar.tools().push_back(Space());
  GenericViewer::toolbar.tools().push_back(Element(torus_select_box));
  torus_select_box.get_entry()->changed.connect(slot(this, &HypersphereViewer::update_current_torus));
  GenericViewer::toolbar.tools().push_back(Space());

  torus_params_window.torus_params_frame.alpha_adjust.value_changed.connect(slot(this, &HypersphereViewer::update_current_torus_stats));
  torus_params_window.torus_params_frame.beta_adjust.value_changed.connect(slot(this, &HypersphereViewer::update_current_torus_stats));
  torus_params_window.torus_params_frame.normalx.activate.connect(slot(this, &HypersphereViewer::update_current_torus_stats));
  torus_params_window.torus_params_frame.normaly.activate.connect(slot(this, &HypersphereViewer::update_current_torus_stats));
  torus_params_window.torus_params_frame.normalz.activate.connect(slot(this, &HypersphereViewer::update_current_torus_stats));
  torus_params_window.torus_params_frame.distance_adjust.value_changed.connect(
							 slot(this, &HypersphereViewer::update_current_torus_stats));
  torus_params_window.torus_params_frame.slicing_adjust.value_changed.connect(
							 slot(this, &HypersphereViewer::update_current_torus_stats));
  torus_params_window.torus_params_frame.dp_rb_circs.pressed.connect(slot(this, &HypersphereViewer::update_current_torus_stats));
  torus_params_window.torus_params_frame.dp_rb_foliation.pressed.connect(slot(this, &HypersphereViewer::update_current_torus_stats));
  torus_params_window.torus_params_frame.dp_rb_mesh.pressed.connect(slot(this, &HypersphereViewer::update_current_torus_stats));
  torus_params_window.torus_params_frame.dp_rb_solid.pressed.connect(slot(this, &HypersphereViewer::update_current_torus_stats));



  GenericViewer::toolbar.tools().push_back(params_window_button);
  params_window_button.clicked.connect(slot(this, &HypersphereViewer::toggle_params_window));
  GenericViewer::toolbar.tools().push_back(delete_torus_button);
  delete_torus_button.clicked.connect(slot(this, &HypersphereViewer::delete_torus));
  add_torus_button=manage(new Gtk::Button("Add new torus"));
  add_torus_button->clicked.connect(slot(&add_torus_window, &AddTorusWindow::show_all));
  GenericViewer::toolbar.tools().push_back(Space());
  GenericViewer::toolbar.tools().push_back(Space());
  GenericViewer::toolbar.tools().push_back(Space());
  GenericViewer::toolbar.tools().push_back(Space());
  GenericViewer::toolbar.tools().push_back(Space());
  GenericViewer::toolbar.tools().push_back(Space());
  GenericViewer::toolbar.tools().push_back(Space());
  GenericViewer::toolbar.tools().push_back(*add_torus_button);

  torus_number=0;
  torus_select_box.set_use_arrows(true);
  torus_select_box.set_use_arrows_always(true);
  //  torus_select_entries.push_back("First string");
  ////  current_combo_entry=NULL;
  current_torus=NULL;
  torus_select_box.set_popdown_strings(torus_select_entries);
  call_update_stats_upon_event=true;

  show_all();
  GenericViewer::initgl();
  draw_gl_context_imp();

  add_torus_window.set_modal(true);
  add_torus_window.set_transient_for(*this);
  add_torus_window.add_torus_request.connect(slot(this, &HypersphereViewer::add_new_torus));
					     /*
					       add_torus_window.get_alpha(),
					       add_torus_window.get_beta(),
					       add_torus_window.get_distance())); */

  /* redraw the screen if torus parameters change */
  redraw_screen_request.connect(slot(this, &HypersphereViewer::draw_gl_context_imp));
  // TODO: In welche Klasse soll das Signal "redraw_torus_request" eingebaut werden und welche kann es dann aufrufen???
  redraw_torus_request.connect(slot(this, &HypersphereViewer::update_current_torus));
}



void HypersphereViewer::add_new_torus() { //double alpha, double beta, double distance) {
  /* Create new torus with the specified data */
  current_torus=new CTorus(SPlane3(add_torus_window.get_normal(), add_torus_window.get_distance()),
			   add_torus_window.get_slicing_number(), add_torus_window.get_draw_method());
  if(current_torus==NULL) {
    cerr << "No memory left for creation of a new torus..." << endl;
    exit(1);
  }

  /* set smooth coloring */
  current_torus->set_coloring(SMOOTH);

  /* Create new name for the torus and increase number of tori */
  char * num=new char(4);
  sprintf(num, "%d", torus_number);
  //  current_torus_name=string(num)+string(" ...another torus");
  current_torus_name=string("Torus No. ")+string(num);
  torus_number++;

  /* Insert the new name in the combo box as well as in the list of existing tori (has to be done first
     since the call of set_popdown_strings triggers an event and calls update_current_torus_stats */
  torus_select_entries.push_back(current_torus_name);
  existing_tori[current_torus_name]=current_torus;
  torus_select_box.set_popdown_strings(torus_select_entries);


  // TODO: Gebraucht?
  //  draw_gl_context_imp();
}




void HypersphereViewer::delete_torus() {
  if(current_torus==NULL) {
    cout << "Kein Eintrag" << endl;
    return;
  }

  /* delete current torus, remove from "existing_tori" and "torus_select_entries",
     set its pointer provisionally to zero and clear the current torus name */
  delete current_torus; // TODO: Was passiert, wenn so ein Torus zerstrt wird (ohne Destruktor??)
  existing_tori.erase(current_torus_name);
  current_torus=NULL;
  torus_select_entries.erase(find(torus_select_entries.begin(),torus_select_entries.end(),current_torus_name));
  current_torus_name="";

  /* set the updated popdown strings in the combo box */
  torus_select_box.set_popdown_strings(torus_select_entries);

  /* any torus left? */
  if(existing_tori.size()!=0) {
    current_torus=(*existing_tori.begin()).second;
    current_torus_name=(*existing_tori.begin()).first;
    
    /* test if the current torus name is as expected the current entry in the combo box */
    if(current_torus_name!=torus_select_box.get_entry()->get_text()) {
      cerr << "Hier ist was falsch mit dem Namen des aktuellen Torus" << endl;
    }
  }
  
  /* hier wird der Name explizit gesetzt */
  torus_select_box.get_entry()->set_text(current_torus_name);


  draw_gl_context_imp();
  //  Viewer::drawgl(); // TODO: Wenn der letzte noch vorhandene Torus gelscht wird, wird er trotzdem noch angezeigt. Ist das ein
  // Fehler hier oder in der Funktion "delete_torus"

  // TODO: Die Tori, die nach dem gelschten in der Combo-Box stehen, umnumerieren und lschen
  // evtl. eine eigene Klasse einfhren, die die Namen, Nummern, Pointer und/oder Referenzen zu
  // den erstellten Tori in einer Liste enthlt und verwaltet.
}


void HypersphereViewer::toggle_params_window() {
  switch(params_window_button.get_active()) {
  case true:
    if(current_torus==NULL) {
      params_window_button.set_active(false);
      return;
    }
    torus_params_window.show_all();
    break;
  case false:
    torus_params_window.hide_all();
    break;
  }
}


/*
void HypersphereViewer::update_current_torus() {
  string tempstr;
  tempstr=torus_select_box.get_entry()->get_text();
  // TODO: Was, wenn kein Eintrag mehr in der Combo ist???
  cout << tempstr << endl;
  if(tempstr=="") {
    cout << "String leer" << endl;
    current_combo_entry=NULL;
    current_torus_iterator=NULL;
    return;
  }
  int pos=atoi(tempstr.c_str());
  cout << pos << endl;
  current_combo_entry=torus_select_entries.begin()+pos-1;
  current_torus_iterator=existing_tori.begin()+pos-1;
  //  current_torus=(*current_combo_entry);

  torus_params_window.set_current_torus(*current_torus_iterator);
}
*/


void HypersphereViewer::update_current_torus() { /* version with map instead of vector */
  /* read the name of the new current torus from the combo box */
  current_torus_name=torus_select_box.get_entry()->get_text();
  if(existing_tori.find(current_torus_name)==existing_tori.end()) {
    //    cout << "Kein Torus mit dem Eintrag " << current_torus_name << " vorhanden" << endl;
    current_torus=NULL;
    torus_params_window.hide_all();
    params_window_button.set_active(false);
    // TODO: Nochwas machen? (Evtl. current_torus_name ndern oder so?)
  } else {
    current_torus=existing_tori[current_torus_name];

    /* Update the values in the parameter window but don't call "update_current_torus_stats()" and don't refresh the screen */
    /*
    cout << "Torus data:" << endl;
    cout << current_torus->get_sect_plane().get_normal() << endl;
    cout << current_torus->get_defining_angles().first << endl;
    cout << current_torus->get_defining_angles().second << endl;
    cout << current_torus->get_drawmethod() << endl;
    cout << current_torus->get_sect_plane().get_distance() << endl << endl;
    */
    call_update_stats_upon_event=false;
    torus_params_window.set_current_torus(current_torus);
    call_update_stats_upon_event=true;
  }

  // TODO: Falls im Textfeld der combo-box ein manueller Eintrag gemacht wird, dessen Gltigkeit berprfen
}



void HypersphereViewer::update_current_torus_stats(void) {
  //  cout << "TODO: Diese Funktion soll die Parameter des aktuellen Torus aendern und den Screen neu zeichnen" << endl;
  ///  if(TorusParamsFrame::torus_data_should_be_updated==false)
  if(call_update_stats_upon_event==false) {
    return;
  }

  current_torus->set_sect_plane(SPlane3(torus_params_window.get_normal(), torus_params_window.get_distance()));
  current_torus->set_num_circles(torus_params_window.get_slicing_number());
  current_torus->set_drawmethod(torus_params_window.get_draw_method());
  //  current_torus->set_coloring(UNI);
  current_torus->set_coloring(SMOOTH);
  draw_gl_context_imp();
  //  redraw_screen_request(); // TODO: Braucht man das ??????
}






gint HypersphereViewer::init_gl_context_imp() {
  /* call parent gl init */
  GenericViewer::init_gl_context_imp();

  //glClearColor(1.0, 1.0, 1.0, 0.0);
  glClearColor(0.0, 0.0, 0.0, 0.0);
  //glDepth(1.0);
  glShadeModel(GL_SMOOTH);
  glEnable(GL_DEPTH_TEST);

  glEnable(GL_CULL_FACE);
  glCullFace(GL_FRONT);
  
  GLfloat lmod_ambient[] = {0.3, 0.3, 0.3, 1.0};
  GLfloat mat_ambient[] = {0.8, 0.8, 0.8, 1.0};
  GLfloat mat_specular[] = {1.0, 1.0, 1.0, 1.0};
  GLfloat mat_emission_front[] = {0.7, 0.0, 0.0, 0.0};
  GLfloat mat_emission_back[] = {0.3, 0.0, 0.0, 0.0};
  GLfloat light_ambient[] = {1.0, 1.0, 1.0, 1.0};
  GLfloat light_color[] = {0.0, 1.0, 1.0, 1.0};
  GLfloat light_position[] = {0.0, 0.0, 1.0, 0.0 };
  
  //    glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
  glLightfv(GL_LIGHT0, GL_SPECULAR, light_color);
  glLightfv(GL_LIGHT0, GL_POSITION, light_position);
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmod_ambient);

  glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
  glMaterialfv(GL_FRONT, GL_EMISSION, mat_emission_front);
  glMaterialfv(GL_BACK, GL_EMISSION, mat_emission_back);
  
  // TODO: Should lighting be enabled???
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  //  glDisable(GL_LIGHTING);
  
  glEnable(GL_AUTO_NORMAL);
  glEnable(GL_NORMALIZE);
  glEnable(GL_MAP2_VERTEX_4);
  
  geradenrichtung[0]=0;
  geradenrichtung[1]=0;
  geradenrichtung[2]=1; //??????????????????????? Achtung: z-Koordinate geht nach vorne!

  return 0;
}



gint HypersphereViewer::draw_gl_context_imp() {
  //  cout << "draw_gl_context_imp() aufgerufen..." << endl;
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glPushMatrix();

  //  glutWireTeapot(0.5); // TODO: Warum erscheint der nicht?


  /*
  //cout << "display" << endl;
  //    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  //    glLoadIdentity();
// calculate correct view vector
    view_vec=Vector3(1,1,view_height);
    Vector3 axis(0,0,-1); // ????????????
    view_vec=q_rotate(view_vec,-rotwinkel*M_PI/180,axis);
    //cout << "View vector:   " << view_vec << endl;
// position the "camera"
    gluLookAt(view_vec[0], view_vec[1], view_vec[2], 0, 0, 0, 0, 0, 1); // ???????????
    glScalef(skalierung, skalierung, skalierung);
    //glRotatef(rotwinkel, 0, 1, 0);  // obsolete
    */

// draw 2 tiny spheres at the center of the big sphere
/*
    glutWireSphere(0.001, 20, 20);
    glutWireSphere(0.00001, 20, 20);
    glutWireCube(1.0);
*/


// draw positioning line
/*
    glPushMatrix();
    glColor3f(0, 0, 1);
    glBegin(GL_LINES);
    glVertex3f( geradenrichtung[0]*3,  geradenrichtung[1]*3,  geradenrichtung[2]*3);
    glVertex3f(-geradenrichtung[0]*3, -geradenrichtung[1]*3, -geradenrichtung[2]*3);
    glEnd();
    glPopMatrix();
*/


// draw 3 spheres on the positive coordinate axes
/*
    // x-Sphere is red
    glPushMatrix();
    glTranslatef(2,0,0);
    glColor3f(1,0,0);
        glutWireSphere(0.1, 10, 10);
    glPopMatrix();
    // y-Sphere is green (?)
    glPushMatrix();
    glColor3f(0,1,0);
    glTranslatef(0,2,0);
      glutWireSphere(0.1, 10, 10);
    glPopMatrix();
    // z-Sphere is blue
    glPushMatrix();
    glColor3f(0,0,1);
    glTranslatef(0,0,2);
      glutWireSphere(0.1, 10, 10);
    glPopMatrix();
*/
    glScalef(0.2,0.2,0.2);


    /* Draw all the torus objects created by the user */
    //    vector<CTorus*>::iterator i;
    map<string, CTorus*>::iterator i;
    int zaehler=0;
    for(i=existing_tori.begin(); i!=existing_tori.end(); i++) {
      if((*i).second!=NULL) {
	//	(*i).second->calc_control_point_parameters();
	//	(*i).second->surf_draw3();
	//	(*i).second->circ_draw();

	/*
	cout << "Drawing method for the current torus " << zaehler << ": " << (*i).second->get_drawmethod() << endl;
	zaehler ++;
	cout << "or else: ";
	switch((*i).second->get_drawmethod()) {
	case CIRCS:
	  cout << "CIRCS " << endl;
	  break;
	case FOLIATED:
	  cout << "FOLIATED " << endl;
	  break;
	case MESH:
	  cout << "MESH " << endl;
	  break;
	case SOLID:
	  cout << "SOLID " << endl;
	  break;
	default:
	  cout << "none" << endl;
	}
	*/
	(*i).second->draw();
      }
    }



    glPopMatrix();


    Viewer::display.swap_buffers(); /* make changes visible */

    return 0;
}
