#ifndef S_PLANE3_H
#define S_PLANE3_H

#include "Vector3.h"
#include "circle3d.h"

/***
 *** This class represents a sectional plane in R^3 which intersects S^2 in a circle.
 *** The plane is specified by its normal vector "normal" and its distance "d" from the origin.
 *** |d| must be <=1 and normal must be <> 0
 ***
 *** TODO: Evtl. ein flag einfuehren, das anzeigt, ob tatsaechlich eine *Schnitt*ebene mit S^2 vorliegt
 ***       oder einfach nur eine ganz normale Ebene.
 ***/

class SPlane3 {
 public:
    SPlane3();
    SPlane3(Vector3 n, double d);
    SPlane3(double alpha, double beta, double d); // the same as before; the normal vector is determined by the angles alpha and beta
                                                  // (between the normal and the coordinate axes)
    SPlane3(Vector3 P1, Vector3 P2, Vector3 P3); // plane through the points P1, P2, P3

    Vector3 get_normal();
    //    void set_normal(Vector3);

    // Kann evtl. gelscht werden
    //    pair<double, double> get_angles_of_normal(); // returns the angles between the normal and the coordinate axes

    void set_distance(double d); // if |d| > 1, then d is set to +/-1 ; TODO: ist das immer noch so und sollte das so sein?
    double get_distance();
    void translate(double d_rel); // moves the plane by d_rel relative to its current position in the direction of "normal"
    Circle3d get_intersecting_circle(); // returns the intersecting circle of the plane with S^2
    Circle3d get_intersecting_circle(Vector3 center, double r); // returns the intersecting circle of the plane with
             // a sphere with radius "r" around "center"

    double distance_from_point(Vector3 P); // Caution: This is a "directed" distance (i.e. it can be negative; the actual distance
                                           // equals abs(distance_from_point)
    Vector3 reflect(Vector3 P);

 private:
    Vector3 normal;
    double distance;
};


inline Vector3 SPlane3::get_normal() {
    return normal;
}

/*
inline void SPlane3::set_normal(Vector3 new_normal) {
    normal=new_normal;
}
*/
#endif
