/*
 *  SpaceEntity.cc
 *
 *  Entity in 3D Space
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "SpaceEntity.h"

SpaceEntity::SpaceEntity():pos(0,0,0),lookat(1,0,0),up(0,1,0)
{
  //just to init the stats with something valid
}

int SpaceEntity::set_pos(Vector3 p){
  Vector3 look_dir=(lookat-p);
  Vector3 new_up;
  /* angle between them */
  double cos=up*look_dir/(up.abs()*look_dir.abs());
  
  /* up must not be parallel to pos-lookat */
  if(!(p==lookat||cos==1||cos==-1)){ 
    /* orthogonalizing up and lookat-dir */
    double ldabs=look_dir.abs();
    for (int i=0;i<=2;i++) new_up[i]=up[i] -cos*look_dir[i]/ldabs;
    
    up=new_up/new_up.abs();
    pos=p;
    up_update(up);
    
  }
  pos_update(pos);
}

int SpaceEntity::set_lookat(Vector3 p){

   Vector3 look_dir=(p-pos);
   Vector3 new_up;
   //angle between them
   double ldabs=look_dir.abs();
   double cos=up*look_dir/(up.abs()*ldabs);

   //up must not be parallel to pos-lookat
   if(!(p==pos||cos==1||cos==-1)){ 
     //orthogonalizing up and look-dir
     for (int i=0;i<=2;i++) new_up[i]=up[i] -cos*look_dir[i]/ldabs;
     up=new_up/new_up.abs();
     lookat=p;
     up_update(up);
   }
   lookat_update(lookat);
   

}

int SpaceEntity::set_up(Vector3 p)
{
  Vector3 look_dir=(lookat-pos);

   double abs=p.abs();
   //normalizing p
   if(abs!=0){
     p=p/abs;
   }
   Vector3 new_up;
   //angle between them
   double ldabs=look_dir.abs();
   double cos=p*look_dir/ldabs;
   //up_dir must not be parallel to pos-lookat and up not be zero
   Vector3 zero(0,0,0);
   if(!(cos==1||cos==-1||p==zero)){ 
     //orthogonalizing up and look_dir
 
     for (int i=0;i<=2;i++) new_up[i]=p[i] -cos*look_dir[i]/ldabs;
     up=new_up/new_up.abs();
   }
   up_update(up);
   
}

SpaceStats SpaceEntity::get_stats()
{
  SpaceStats res;
  res.pos=pos;
  res.lookat=lookat;
  res.up=up;
  return res;

}



