#include "circle3d.h"
//#include "quat_math.h"

Circle3d::Circle3d() {
    center=Vector3(0,0,0);
    radius=1;
    normal=Vector3(0,0,1);
    for(int i=0; i<3; i++) {
	col_vec[i]=1.0;
    }
}


Circle3d::Circle3d(const Circle3d &rhs) {
    center=rhs.center;
    radius=rhs.radius;
    normal=rhs.normal;
    for(int i=0; i<3; i++) {
	col_vec[i]=rhs.col_vec[i];
    }
}


Circle3d::Circle3d(Vector3 cen, double r, Vector3 n, double R, double G, double B) {
    center=cen;
    radius=fabs(r);
    normal=n;
    col_vec[0]=R;
    col_vec[1]=G;
    col_vec[2]=B;
}


Circle3d::Circle3d(Vector3 cen, double r, Vector3 n, double * RGB) {
    center=cen;
    radius=r;
    normal=n;
    for(int i=0; i<3; i++) {
	col_vec[i]=RGB[i];
    }
}

//TODO: This function can be programmed more efficiently (eliminate the if-statement)
//      It might also be reprogrammed by only using the function "get_point_from_parameter" !!!
//      Another alternative would be to implement a class for the maps used to move
//      the complex unit circle to the given circle
//
// The variable t is a parameter indicating where to place the first point on the circle
vector<Vector3>& Circle3d::get_points(unsigned int num, double t) {
    vector<Vector3> *points=new vector<Vector3>;
    Vector3 rot_axis, x_vec, move;
    double angle, slangle=2*M_PI/num;

    t=t-floor(t); // normalizing t to the intervall [0,1[
    double t0=t*2*M_PI;
//    if(fabs(normal[1])<fabs(normal[0])) {
	x_vec=Vector3(1,0,0);
	rot_axis=x_vec^normal;
	angle=v_angle(x_vec,normal); // angle is in radians
	// if the circle doesn't need to be rotated, set rot_axis accordingly
	if(rot_axis==Vector3(0,0,0)) {
	    rot_axis=Vector3(0,1,0);
	    angle=0;
	}


	for(int i=0; i<num; i++) {
	    x_vec=Vector3(0, radius*cos(i*slangle+t0), radius*sin(i*slangle+t0));
	    (*points).push_back(q_rotate(x_vec, angle, rot_axis)+center);
	}


/*    } else {
	temp_vec=Vector3(1,0,0);
	rot_axis=temp_vec^normal;
	angle=v_angle(temp_vec,normal)*180/M_PI;
	
	for(int i=0; i<num; i++) {
	    temp_vec=Vector3(0, radius*cos(i*slangle), radius*sin(i*slangle));
	    (*points).push_back(q_rotate(temp_vec, angle, rot_axis)+move);
	}
    }
*/

    return *points;
}


// The given circle is parametrized by a variable t varying between 0 and 1.
// t is first normalized to the interval [0,1[
// The point being returned is determined by first calculating the point P on the complex unit circle
// which is given by t (beginning at 1+0*i and proceeding counterclockwise). Then the following maps
// are applied to P: First a rotation and afterwards a translation that maps the complex unit circle
// to the given circle.
Vector3 Circle3d::get_point_from_parameter(double t) {
    double t0=t-floor(t);
    Vector3 point;
    Vector3 rot_axis, x_vec, move;
    double angle, slangle=2*M_PI*t0;

//    if(fabs(normal[1])<fabs(normal[0])) {
	x_vec=Vector3(1,0,0);
	rot_axis=x_vec^normal;
	angle=v_angle(x_vec,normal);
	// if the circle doesn't need to be rotated, set rot_axis accordingly
	if(rot_axis==Vector3(0,0,0)) {
	    rot_axis=Vector3(0,1,0);
	    angle=0;
	}

	x_vec=Vector3(0, radius*cos(slangle), radius*sin(slangle));
	point=q_rotate(x_vec, angle, rot_axis)+center;

	return point;
}



void Circle3d::set_color(double * RGB) {
    for(int i=0; i<3; i++) {
	col_vec[i]=RGB[i];
    }
}


void Circle3d::set_color(double R, double G, double B) {
    col_vec[0]=R;
    col_vec[1]=G;
    col_vec[2]=B;
}



Vector3 Circle3d::get_intersection(bool first_or_second, Vector3 plane_normal) {
  Vector3 temp=plane_normal^(normal.normalize());
  // TODO: Enhance error handling in the following case
  if(temp==Vector3(0,0,0)) {
    cerr << "The circle lies in the x,y-plane, thus there are more than two intersecting points..." << endl;
    return temp;
  }
  if(first_or_second==true)
    return center+temp.normalize()*radius;
  else 
    return center-temp.normalize()*radius;
}



pair<Vector3, Vector3> Circle3d::get_directed_intersection(Vector3 dir) {
  Vector3 point1, point2, direction=dir.normalize()*radius;
  point1=center+direction;
  point2=center-direction;
  return pair<Vector3, Vector3>(point1,point2);
}




double* Circle3d::get_color() {
    double *ar=new double[3];
    for(int i=0; i<3; i++) {
	ar[i]=col_vec[i];
    }
    return ar;
}


void Circle3d::draw() {
    //cout << "calling draw method" << endl;
    glPushMatrix();
    //cout << "Center coordinates: " << center[0] << " " << center[1] << " " << center[2] << endl;
    //glLoadIdentity();
    glTranslatef(center[0],center[1],center[2]);
    Vector3 up(0,0,1);
    Vector3 rot_axis=up^normal;
    double ang=v_angle(up,normal)*180/M_PI;
    glRotatef(ang, rot_axis[0], rot_axis[1], rot_axis[2]);

    GLUquadricObj *myquad=gluNewQuadric();
    gluQuadricDrawStyle(myquad, GLU_LINE);
    glColor3dv(get_color());
    //    gluDisk(myquad, radius*0.9, radius*1.1, 60, 30);
    gluDisk(myquad, radius, radius, 60, 1);
    gluDeleteQuadric(myquad);
    glPopMatrix();
}


Circle3d Circle3d::pre_image(Vector3 &half_line) {
/***
 *** Attention: In David Lyons' article the coordinates in the hopf map are permuted!!!
 *** How does this affect the calculations in this function???
 *** Don't panic: Doesn't have a visible impact !?!
 ***/

    // P is the point on S^2 whose preimage is to be calculated
    Vector3 P=half_line.normalize();

    // calculate the quaternion r=(0,r2,r3,r4) which determines the linear map mapping the complex unit circle to
    // the desired preimage
    double factor=1/sqrt(2*(1+P[0]));
    double r2=factor*(1+P[0]);
    double r3=factor*P[1];
    double r4=factor*P[2];
    // d = distance from Z(0,0,0,1) to the plane E of h^(-1)(P)
    if(r2==0) {
	cerr << "TODO: What is the image of a circle through Z?" << endl;
	exit(1);
    }

    /***
    // Calculate the center of the stereographic projection of the preimage 
    Quat M(0,0,0,1);
    Quat n(r3,r4,0,-r2);
    M=M+n*(1/r2);
    if(M.get_vec()[3]>0.001) {
	cerr << "Error: Center of projected circle does not lie in {x4=0}-space:  x4=" << M.get_vec()[3] << endl;
        exit(1);
    } // else {
    //   cout << "Alles im gruenen Bereich: M liegt im {x4=0}-Raum:  M[4]=" << M.get_vec()[4] << endl;
    // }
    Vector3 center(M.get_vec()[0],M.get_vec()[1],M.get_vec()[2]); //center of the projected circle
    ***/

    //TODO: Eigentlich kann man die Koordinaten von "center" auch direkt angeben:
    //      Daher habe ich mal den ganzen vorherigen Absatz auskommentiert.
    //      Wenn man allerdings die Hyperkugel vor der stereographischen Projektion noch drehen will, dann
    //      Kann es sein, da die Berechnungen komplizierter werden und man "center" nicht mehr so einfach
    //      ausrechnen kann. Wie mu es dann gemacht werden???
    Vector3 center(r3/r2, r4/r2, 0);

    // normal vector of the plane of the projected circle
    // TODO: How did I find out how to determine this?
    Vector3 v1(0,r2,r3);
    Vector3 v2(-r2,0,r4);
    Vector3 normal=v1^v2;

    // TODO: is d or 1/d the radius of the projected circle?
    Circle3d proj_circle(center, 1/fabs(r2), normal);


    return proj_circle;
}


