#ifndef CIRCLE3D_H
#define CIRCLE3D_H

#include <math.h>
#include <vector>
#include <GL/glut.h>
#include "Vector3.h"
//#include "SPlane3.h"
#include "QuatMath.h"


/***
 *** This class represents a circle in 3-dimensional space. It is specified by its center, its radius
 *** and a normal vector to the plane of the circle
 ***/

class Circle3d {
 public:
    Circle3d();
    Circle3d(const Circle3d &rhs);
    // TODO: Check if color information provided by RGB is correct!
    Circle3d(Vector3 center, double radius, Vector3 normal, double R=1.0, double G=1.0, double B=1.0);
    Circle3d(Vector3 center, double radius, Vector3 normal, double * RGB);
    Vector3 get_center();
    void set_center(Vector3 cen);
    double get_radius();
    Vector3 get_normal(); // returns normal vector of the plane of the circle
    Vector3 get_unit_normal(); // as get_normal(), but return vector ist normalized
    void turn_normal(); // changes the sign of the normal vector (the function "draw_circle_strip" needs this)
    // returns a vector of num points equally distributed on the circle
    vector<Vector3>& get_points(unsigned int num, double t=0);
    //double get_point_parameter(
    Vector3 get_point_from_parameter(double t); // the given circle is parametrized by a single variable t varying
                                                // between 0 and 1; for details see "circle3d.cc"
    Vector3 get_intersection(bool first_or_second=true, Vector3 plane_normal=Vector3(0,0,1)); // returns one of the two
    // intersecting points of the circle with the plane whose normal vector is given and which passes through the origin
    // (default: x,y-plane); TODO: Maybe return *both* intersecting points (in an array --> or in a Pair???)
    pair<Vector3, Vector3> get_directed_intersection(Vector3 dir); // returns the two points obtained by intersecting the circle
    // with a line passing through its center and pointing in the direction of dir; the point in the direction of dir
    // is returned first

    void set_color(double * RGB);
    void set_color(double R, double G, double B);
    double* get_color(); // returns a vector of pointers to doubles containing the color information of the circle

    void draw();
    static Circle3d pre_image(Vector3 &half_line); // returns the circle in R^3 which is the stereographic
    // projection of the pre_image under the Hopf map of a point P. P lies on S^2 and is specified
    // by a half-line whose direction is given as an argument (the given vector "half_line" is simply normalized)


 private:
    double radius;
    Vector3 center;
    Vector3 normal;
    double col_vec[3];
};


inline Vector3 Circle3d::get_center() {
    return center;
}

inline void Circle3d::set_center(Vector3 cen) {
    center=cen;
}

inline double Circle3d::get_radius() {
    return radius;
}

inline Vector3 Circle3d::get_normal() {
    return normal;
}


inline Vector3 Circle3d::get_unit_normal() {
    return normal.normalize();
}


inline void Circle3d::turn_normal() {
    for(int i=0; i<3; i++) {
	normal[i]=-normal[i];
    }
}


#endif
