/*
 *  Gtk_GroupEntry.h
 *
 *  Group of entry widgets
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#ifndef GTK_GROUPENTRY_H
#define GTK_GROUPENTRY_H

#include <gtk--/box.h>
#include <gtk--/entry.h>
#include <gtk--/table.h>
#include <gtk--/label.h>
#include <gtk--/frame.h>

#include <vector>
#include <list>


/** Gtk Group Entry class.
    This class provides a group of Gtk Entries that are surrounded
    by a frame and stored in a vector for easy access. 
    It includes a signal rising when
    an Entry is activated, providing the vector index of the 
    concerning entry. 
*/
class GroupEntry : public Gtk::Frame {
  
  vector<Gtk::Label* > ll;
  vector<Gtk::Entry* > el;
  Gtk::HBox hbox;

  void on_entry_activated(int i);

public:

  /** Constructor. The first string in strlist is the frame label the 
      next ones are the labels next to the entries. 
  */
  GroupEntry( list<char* >& strlist, int entry_width = 80, int height = 22, int length = 6);

  /** rising when activated and providing vector index */
  SigC::Signal1<void, int> entry_activated;

  /** get the text value of entry with index i */
  string get_entry( int i) { return el[i]->get_text(); }

  /** set the text value of entry with index i */
  void set_entry( int i, string s) { el[i]->set_text(s); }

};



#endif
