/*
 *  Gtk_LightControl.cc
 *
 *  Lighting Control widget
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "Gtk_LightControl.h"

using SigC::bind;
using SigC::slot;


LightControl::LightControl() :
  Viewer("Light Control"),
  rotlscene_b("Scene"),
  rotlight_b("Light"),
  light_pix("./pixmaps/light.xpm"),
  mode(1),
  display_list(false),
  list_index(-1),
  raster(10),
  finish_render(true)
{

  rotlscene_b.toggled.connect( bind<int>(slot(this, &LightControl::on_button_toggled), 1));
  rotlight_b.toggled.connect( bind<int>(slot(this, &LightControl::on_button_toggled), 2));

  rotlscene_b.set_active(true);
  
  main_button.add(light_pix);
  
  Viewer::upper_hbox.pack_start(rotlscene_b);
  Viewer::upper_hbox.pack_start(rotlight_b);

  display.set_usize(200, 200);
  
  set_usize(200, 240);
  
  active = 0;

}

void LightControl::set_display_list( gint i) {
  list_index = i;
  display_list = true;
}

gint LightControl::draw_gl_context_imp() {

  if( display_list && list_index > -1 && finish_render) {
    glCallList( list_index);
    glPushMatrix();
    glTranslatef( gl_light_pos[0], gl_light_pos[1], gl_light_pos[2]);
    draw_flashlight();
    glPopMatrix();
  }
  else
    glutSolidSphere(0.3,50,50);  

}


void LightControl::on_button_toggled( int id) {
  if (id==1 && rotlscene_b.get_active()) 
    {
      rotlight_b.set_active(false);
      mode = 1;
    }


  if (id==2 && rotlight_b.get_active())
    {
      rotlscene_b.set_active(false);
      mode = 2;
    }

  if ( !rotlscene_b.get_active() && !rotlight_b.get_active())
    {
      if (id==1)  rotlscene_b.set_active(true);
      else  rotlight_b.set_active(true);
    }

}

void LightControl::set_active(gboolean state)
{
  if (state) 
    {
      active=1;
      show_all();
    }
  else
    {
      active=0;
      hide_all(); 
    }

}


/*i dont want the widget to be destroyed when closed but to go inactive and hide*/
int LightControl::delete_event_impl(GdkEventAny* event)
{

  main_button.set_active(false);

  /* returnig true means it wont be destroyed */
  return true;
}


gint LightControl::gl_motion_notify_handler_imp(GdkEventMotion* event) {
  
  if( mode == 1) 
    Viewer::gl_motion_notify_handler_imp(event);
  else if( mode == 2) {
    
    gint x,y;
      
    GdkModifierType state;
      
    if (event->is_hint)
      gdk_window_get_pointer (event->window, &x, &y, &state);
    else {
	x = (gint) event->x;
	y = (gint) event->y;
	state =(GdkModifierType) event->state;
    }
      
    if ((state & GDK_BUTTON1_MASK) && gl_mouse1_pressed) {
	
      gl_rot_x = y - gl_mouse_start_y;
      gl_mouse_start_y = y;
      gl_rot_y = -( x - gl_mouse_start_x);
      gl_mouse_start_x = x;
      
      Vector3 camera_dir = camera.pos - camera.lookat;
      gl_light_pos = q_rotate( gl_light_pos, -gl_rot_y/200, camera.up);
      gl_light_pos = q_rotate( gl_light_pos, -gl_rot_x/200, camera_dir^camera.up/camera_dir.abs());
      
      gl_rot_x = gl_rot_y = 0;
 	 
      Viewer::updategl();
      Viewer::drawgl();
      
    }


    if ((state & GDK_BUTTON2_MASK) && gl_mouse2_pressed) {

      cout << "2" << endl;
    }
    if ((state & GDK_BUTTON3_MASK) && gl_mouse3_pressed) {
      
    }
  }
  return 0;
}


gint LightControl::gl_button_release_handler_imp(GdkEventButton* event){
  if (event->button==1)  {
      light_set();
  }

  Viewer::gl_button_release_handler_imp(event);
}



void LightControl::draw_flashlight() {

}
