/*
 *  Gtk_LightControl.h
 *
 *  Lighting Control widget
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */



#ifndef GTK_LIGHTCONTROL_H
#define GTK_LIGHTCONTROL_H


#include "../Viewer.h"


class LightControl : public Viewer {

  Gtk::ToggleButton rotlscene_b;
  Gtk::ToggleButton rotlight_b;
  
  Gtk::Pixmap light_pix;

  int mode;
  bool display_list;
  gint list_index;

  void on_button_toggled(int i);

 public:
  
  LightControl();
  
  bool finish_render;

  /** Light set signal. This signal is emitted when the light in the main window 
      can be reset. */
  SigC::Signal0<void> light_set; 

  int raster;
  void draw_flashlight();

  Vector3 get_light() { return gl_light_pos;}

  /** This is the button which is to inserted into the main programms toolbar.
      It emitts the usual clicked signal, and the main programm can decide 
      wether to acivate/deactivate and show/hide the window or not.
      (by calling set_active)*/
  Gtk::ToggleButton main_button;
  
  /** If state is true than the window is showed and active set to 1.
      Otherwise it is hidden and active set to zero.*/
  void set_active(gboolean state);

  /* use display list to visualize lighting */
  void set_display_list( gint i);

  /** Is the window activated? */
  char active;

 /** We don't want the widget to be destroyed when closed but to go inactive 
     and hide, so we override this method. */
  virtual int delete_event_impl(GdkEventAny* event);

  virtual gint gl_motion_notify_handler_imp(GdkEventMotion* event); 
  virtual gint gl_button_release_handler_imp(GdkEventButton* event); 

  virtual gint draw_gl_context_imp();
  

};

#endif
  
